from __future__ import unicode_literals
import six
import unittest

from examples.example import User, Entry

try:
    import pymongo
    PYMONGO_CLIENT = pymongo.MongoClient()
except:
    PYMONGO_CLIENT = False


class HelperFuncionsTestCase(unittest.TestCase):
    def setUp(self):
        User.get_collection().remove()
        Entry.get_collection().remove()

    def print_stuff(self, added, *args):
        if User == type(added):
            print('{0.name} joined, following: {0.following}'.format(added))
        elif Entry == type(added):
            print('{0.name} > "{1.title}", {1.categories}'.format(args[0], added))
        elif six.text_type == type(added):
            print('{1.name} commented: "{0}"'.format(added, args[0]))

    @unittest.skipUnless(PYMONGO_CLIENT, 'pymongo not installed or connection refused')
    def test_z_general_functionality(self):
        """Use case demo & test
        """
        print('\n')
        batman = User(name='Batman')
        batman.insert()
        batman.follow('crime', 'energy', 'arkham', 'gadgets')
        self.print_stuff(batman)
        selina = User(name='Selina Kyle', following=['gadgets'])
        selina.insert()
        selina.follow('billionaires')
        self.print_stuff(selina)
        news = User(name='Gotham News')
        news.insert()
        self.print_stuff(news)
        # Lets add some entries
        e1, e2, e3 = [
            news.add_entry('Arkham blocks city sewage system', categories=['arkham', 'lol']),
            news.add_entry('Rich people ball by this new chick in town', categories=['billionaires']),
            news.add_entry('Jetpack Belt released', categories=['gadgets'])
        ]
        for e in (e1, e2, e3):
            self.print_stuff(e, news)
        self.assertEqual(3, news.get_entries().count())
        self.assertTrue(1 == e1.get_followers().count() and
                        batman == e1.get_followers()[0])
        self.assertTrue(1 == e2.get_followers().count() and
                        selina == e2.get_followers()[0])
        e3followers = sorted(e3.get_followers(), key=lambda u: u['name'])
        self.assertTrue(2 == len(e3followers))
        self.assertEqual([batman, selina], e3followers)
        # some comments
        self.print_stuff(e1.add_comment('Those damn nutjobs!', batman), batman)
        self.print_stuff(e2.add_comment('Hawt and looking for a date to this', selina), selina)
        self.print_stuff(e3.add_comment('OMG WANT!', selina), selina)
        self.print_stuff(e3.add_comment('Aww Yea! Suck it Clark!', batman), batman)
        self.assertEqual(2, len([c for c in batman.get_comments()]))
        self.assertEqual(2, len([c for c in selina.get_comments()]))
        self.assertEqual(0, len([c for c in news.get_comments()]))
        for entry in batman.get_comments(with_entries=True, sort=[('_id', pymongo.DESCENDING)]):
            if entry.title.startswith('Arkham'):
                self.assertEqual(1, len(entry.comments))
            elif entry.title.startswith('Jetpack'):
                self.assertEqual(2, len(entry.comments))
            else:
                raise AssertionError('Extra entry received')



class NanomongoError(Exception):
    """Base nanomongo error"""


class ValidationError(NanomongoError):
    """Raised when a field fails validation"""


class ExtraFieldError(NanomongoError):
    """Raised when a document has an undefined field"""


class ConfigurationError(NanomongoError):
    """Raised when a required value found to be not set during
    operation, or a Document class is registered more than once
    """


class IndexMismatchError(NanomongoError):
    """Raised when a defined index does not match defined fields"""


class UnsupportedOperation(NanomongoError):
    """Raised when an unsupported opperation/parameters are used"""


class DBRefNotSetError(NanomongoError):
    """Raised when a DBRef getter is called on not-set DBRef field"""



import copy
import datetime
import math
import unittest

import six

from bson import DBRef, ObjectId

from nanomongo.field import Field
from nanomongo.errors import ValidationError


def wrap(*args, **kwargs):  # to wrap Field() declarations
    def wrapper():
        Field(*args, **kwargs)
    return wrapper


class FieldTestCase(unittest.TestCase):

    def test_bad_types(self):
        """Test Field definitions with invalid types"""
        class RandomClass(object):
            pass

        class TypeSubclass(type):
            pass

        self.assertRaises(TypeError, Field, *(RandomClass,))
        self.assertRaises(TypeError, Field, *(TypeSubclass,))
        self.assertRaises(TypeError, Field, *(1,))

    def test_good_types(self):
        """Test Field definitions with valid types"""
        types = copy.copy(Field.allowed_types)
        [Field(t) for t in types]

    def test_field_def_arguments(self):
        """Test Field definition arguments"""
        valid_defs = [
            wrap(bool, required=True), wrap(bool, required=False),
            wrap(bool, default=True), wrap(bool, default=False),
            wrap(six.binary_type, default=b'L33t'), wrap(six.text_type, default=six.u('L33t')),
            wrap(list, default=[]), wrap(six.text_type, default=None, required=False),
            wrap(datetime.datetime, auto_update=True),
            wrap(DBRef, document_class='L33tClass'),
        ]
        invalid_defs = [
            wrap(), wrap(bool, default=1), wrap(dict, default=None),
            wrap(six.text_type, required=1), wrap(six.binary_type, default=six.u('')),
            wrap(six.text_type, default=b'', required=False),
            wrap(six.text_type, default='', bad_kwarg=True), wrap(int, auto_update=True),
            wrap(datetime.datetime, auto_update='bad value'),
            wrap(DBRef, document_class=dict),
        ]
        [wrapped() for wrapped in valid_defs]
        [self.assertRaises(TypeError, wrapped) for wrapped in invalid_defs]

    def test_field_validators(self):
        """Test some validators"""
        bad_dicts = [
            {'foo.bar': 42}, {'$foo': 42}, {'foo': {'bar.foo': 42}},
            {'foo': {'$bar': 42}},
        ]
        for dct in bad_dicts:
            self.assertRaises(ValidationError, Field(dict).validator, *(dct,))



import unittest

from nanomongo.field import Field
from nanomongo.document import Nanomongo


class NanomongoTestCase(unittest.TestCase):
    def test_nanomongo_inits(self):
        self.assertRaises(TypeError, Nanomongo, *(1,))
        self.assertRaises(TypeError, Nanomongo, **{'foo': 'bar'})
        self.assertRaises(TypeError, Nanomongo, **{'fields': 'bar'})
        self.assertRaises(TypeError, Nanomongo.from_dicts)
        self.assertRaises(TypeError, Nanomongo.from_dicts, *(1,))
        fields_1 = {'foo': Field(str)}
        fields_2 = {'foo': Field(str), 'bar': Field(int)}
        nano_1 = Nanomongo(fields=fields_1)
        nano_2 = Nanomongo.from_dicts(fields_1, fields_2)
        self.assertEqual(['foo'], nano_1.list_fields())
        self.assertEqual(['bar', 'foo'], nano_2.list_fields())



import unittest

from nanomongo.document import BaseDocument, Field
from nanomongo.errors import ConfigurationError

try:
    import pymongo
    import bson
    pymongo.MongoClient()
    PYMONGO_OK = True
except:
    PYMONGO_OK = False


class PY3SugarTestCase(unittest.TestCase):
    def test_class_kwargs(self):
        """Test keyword arguments given at class definition eg.
        client, dot_notation etc.
        """
        class Doc(BaseDocument, dot_notation=True):
            foo = Field(str)

        d = Doc()
        self.assertRaises(AttributeError, lambda: d.foo)
        d.foo = 'bar'
        self.assertTrue('foo' in d and 'bar' is d.foo)

    @unittest.skipUnless(PYMONGO_OK, 'pymongo not installed or connection refused')
    def test_document_register_configuration(self):
        """Test MongoDB client/db/collection config as class kwargs
        """
        client = pymongo.MongoClient()

        class Doc(BaseDocument, client=client, db='nanomongotest'):
            pass

        class Doc2(Doc, dot_notation=True, client=client, db='nanomongotest',
                   collection='othercollection'):
            pass

        self.assertTrue(Doc.nanomongo.registered)
        self.assertEqual(client.nanomongotest.doc, Doc.get_collection())
        self.assertRaises(ConfigurationError, Doc.register,
                          **{'client': client, 'db': 'nanomongotest'})
        self.assertEqual(client.nanomongotest.othercollection, Doc2.get_collection())



import copy
import datetime
import types
import unittest
import sys

import six

from nanomongo.field import Field
from nanomongo.document import BaseDocument, Index
from nanomongo.errors import *

try:
    import pymongo
    import bson
    pymongo.MongoClient()
    PYMONGO_OK = True
except:
    PYMONGO_OK = False


class DocumentTestCase(unittest.TestCase):
    def test_document_bad_field(self):
        """Test document definition with bad Field def"""

        def bad_field():
            class Doc(BaseDocument):
                foo = Field()

        def bad_field_default():
            class Doc(BaseDocument):
                foo = Field(six.text_type, default=1)

        def bad_field_name():
            class Doc(BaseDocument):
                nanomongo = Field(int)

        for func in (bad_field, bad_field_default, bad_field_name):
            self.assertRaises(TypeError, func)

    def test_document_bad_init(self):
        """Test incorrect document init"""

        class Doc(BaseDocument):
            pass
        self.assertRaises(TypeError, Doc, *(1,))
        self.assertRaises(ExtraFieldError, Doc, *({'foo': 'bar'},))
        self.assertRaises(ExtraFieldError, Doc, **{'foo': 'bar'})

        class NewDoc(Doc):
            foo = Field(dict)
        self.assertRaises(ValidationError, NewDoc, *({'foo': {'$bar': 42}},))
        self.assertRaises(ValidationError, NewDoc, **{'foo': {'bar.foo': 42}})

    def test_dot_notation_no_conflict(self):
        """Test that :class:`~nanomongo.util.DotNotationMixin` does not
        interfere when attribute access on a non-field name is UnsupportedOperation
        """
        class Doc(BaseDocument):
            dot_notation = True
            foo = Field(six.text_type)

        d = Doc()
        self.assertFalse(hasattr(Doc, 'dot_notation') or hasattr(d, 'dot_notation'))
        self.assertRaises(AttributeError, lambda: d.foo)
        d.foo = 42
        self.assertTrue('foo' in d)
        # attribute set/get works as usual for names that are not fields
        self.assertRaises(AttributeError, lambda: d.bar)
        d.bar = 'bar value'
        self.assertTrue('bar' not in d)  # not in the data that we'll save
        self.assertEqual('bar value', d.bar)
        del d.bar
        self.assertRaises(AttributeError, lambda: d.bar)
        del d['foo']

        def f():
            del d.foo
        self.assertRaises(AttributeError, f)

    def test_document(self):
        """Test document definition, initialization, setting and getting
        attributes, validation
        """

        class Doc(BaseDocument):
            # dot_notation = False
            bool_a, bool_b = Field(bool), Field(bool)
            foo = Field(int, default=42)
            bar = Field(six.text_type, default=None, required=False)

        class Doc2(Doc):
            dot_notation = True
            foo = Field(dict)
            zoo = Field(list)

        d = Doc(bool_a=True)
        self.assertEqual(d, {'bool_a': True, 'foo': 42, 'bar': None})
        # bool_b not set and required
        self.assertRaises(ValidationError, d.validate_all)
        # dot_notation not active
        d.bool_b = False
        self.assertRaises(ValidationError, d.validate_all)
        d['bool_b'] = True
        d.validate_all()
        # Doc2 has field 'foo' chaged to type dict, can't create new Doc2 from Doc
        self.assertRaises(ValidationError, Doc2, *(d,))
        self.assertRaises(ValidationError, Doc2, **d)
        # this type change will allow Doc2 cast, but fail Doc validation
        d['foo'] = {'L33t': 42}
        self.assertRaises(ValidationError, d.validate_all)
        dd = Doc2(d)
        self.assertTrue(dd == Doc2(**d) ==
                        Doc2(bool_a=True, bool_b=True, foo={'L33t': 42}, bar=None))
        # dd (Doc2) still has required field 'zoo' unset
        self.assertRaises(ValidationError, dd.validate_all)
        # dd has dot_notation
        dd.zoo = [3.14159265]
        dd.validate_all()
        dd.undefined = 'undefined field value'
        dd.validate_all()  # passes because undefined Fields bypass __setattr__
        dd['undefined'] = 'undefined field value'
        self.assertRaises(ValidationError, dd.validate_all)

    def test_document_dir(self):
        """Test __dir__ functionality"""
        class Doc(BaseDocument):
            foo = Field(six.text_type)

        class Doc2(Doc):
            bar = Field(six.text_type, required=False)

        dir_base = BaseDocument()
        dir_doc = Doc()
        dir_doc2 = Doc2()
        base_dir = dir(dir_base)
        doc_dir = sorted(dir(dir_base) + Doc.nanomongo.list_fields())
        doc2_dir = sorted(doc_dir + ['bar'])


class ClientTestCase(unittest.TestCase):
    def test_document_cient_bad(self):
        def bad_client():
            class Doc(BaseDocument):
                client = ''

        def bad_db():
            class Doc(BaseDocument):
                db = 1234

        def bad_col():
            class Doc(BaseDocument):
                dot_notation = True
                collection = 3.14159265

        def db_before_client():
            class Doc(BaseDocument):
                db = 'nanomongotest'

        [self.assertRaises(TypeError, func) for func in (bad_client, bad_db, bad_col)]
        self.assertRaises(ConfigurationError, db_before_client)

    @unittest.skipUnless(PYMONGO_OK, 'pymongo not installed or connection refused')
    def test_document_client(self):
        """Pymongo: Test correct client input and document configuration"""
        mclient = pymongo.MongoClient()

        class Doc(BaseDocument):
            client = mclient
            db = 'nanomongotest'
            dot_notation = True
            foo = Field(six.text_type)

        class Doc2(Doc):
            client = mclient
            db = 'nanomongotest'
            collection = 'othercollection'
            dot_notation = True
            bar = Field(int)

        class Doc3(Doc):
            bar = Field(float)

        d = Doc()
        dd = Doc2()
        ddd = Doc3()
        self.assertEqual(d.nanomongo.client, dd.nanomongo.client)
        self.assertEqual(mclient['nanomongotest'], d.nanomongo.database)
        self.assertEqual(mclient['nanomongotest'], dd.nanomongo.database)
        self.assertEqual('doc', d.nanomongo.collection)
        self.assertEqual('othercollection', dd.nanomongo.collection)
        self.assertNotEqual(d.nanomongo.client, ddd.nanomongo.client)
        self.assertFalse(hasattr(dd, 'client') or hasattr(dd, 'db') or
                         hasattr(dd, 'collection'))

    @unittest.skipUnless(PYMONGO_OK, 'pymongo not installed or connection refused')
    def test_document_configuration(self):
        """Pymongo: Test document misconfiguration eg. client, db, collection"""
        mclient = pymongo.MongoClient()

        class Doc(BaseDocument):
            pass

        class Doc2(BaseDocument):
            client = mclient

        class Doc3(BaseDocument):  # autoset
            client = mclient
            db = 'nanotestdb'

        self.assertRaises(ConfigurationError, Doc.get_collection)
        self.assertRaises(ConfigurationError, Doc2.get_collection)
        Doc3.get_collection()
        Doc3.nanomongo.collection = ''
        self.assertRaises(ConfigurationError, Doc3.get_collection)
        # register
        self.assertRaises(ConfigurationError, Doc.register)
        self.assertRaises(ConfigurationError, Doc.register, **{'client': mclient})
        self.assertRaises(TypeError, Doc.register,
                          **{'client': mclient, 'db': mclient['nanotestdb']})
        self.assertFalse(Doc.nanomongo.registered)
        Doc.register(client=mclient, db='nanotestdb')
        self.assertTrue(Doc.nanomongo.registered)
        self.assertRaises(ConfigurationError, Doc.register,
                          **{'client': mclient, 'db': 'nanotestdb'})
        m_count = len(Doc.get_collection().database.outgoing_copying_manipulators)
        self.assertEqual(1, m_count)

        Doc2.register(client=mclient, db='nanotestdb', collection='doc2_collection')


class MongoDocumentTestCase(unittest.TestCase):
    def setUp(self):
        pymongo.MongoClient().drop_database('nanotestdb')

    @unittest.skipUnless(PYMONGO_OK, 'pymongo not installed or connection refused')
    def test_insert_find(self):
        """Pymongo: Test document insert, find, find_one"""
        client = pymongo.MongoClient()

        class Doc(BaseDocument):
            dot_notation = True
            foo = Field(six.text_type)
            bar = Field(int, required=False)
        Doc.register(client=client, db='nanotestdb')

        self.assertEqual(None, Doc.find_one())
        d = Doc(foo=six.u('foo value'))
        d.bar = 'wrong type'
        self.assertRaises(ValidationError, d.insert)
        d.bar = 42
        self.assertTrue(d.insert())
        self.assertEqual(0, Doc.find({'foo': 'inexistent'}).count())
        self.assertEqual(1, Doc.find({'foo': 'foo value'}).count())
        self.assertEqual(d, Doc.find_one())

    @unittest.skipUnless(PYMONGO_OK, 'pymongo not installed or connection refused')
    def test_partial_update(self):
        """Pymongo: Test partial atomic update with save"""
        client = pymongo.MongoClient()

        class Doc(BaseDocument):
            dot_notation = True
            foo = Field(six.text_type)
            bar = Field(int, required=False)
            moo = Field(list)
        Doc.register(client=client, db='nanotestdb')

        d = Doc(foo=six.u('foo value'), bar=42)
        self.assertRaises(ValidationError, d.save)  # no _id yet
        d['_id'] = bson.objectid.ObjectId()
        self.assertRaises(ValidationError, d.save)  # _id manually set
        self.assertRaises(ValidationError, d.insert)  # missing field moo
        d.moo = []
        self.assertEqual(d._id, d.insert())
        del d.bar  # unset
        d.save()
        self.assertEqual(d, Doc.find_one({'_id': d._id}))
        d.foo = six.u('new foo')
        d['bar'] = 1337
        d.moo = ['moo 0']
        d.save(atomic=True)
        self.assertEqual(d, Doc.find_one({'_id': d._id}))
        d.moo = []
        del d['bar']
        d.save()
        self.assertEqual(d, Doc.find_one({'_id': d._id}))
        d['extra_field'] = 'fail'
        self.assertRaises(ValidationError, d.save)
        del d['extra_field']
        d.save()
        d.bar = 'wrong type'
        self.assertRaises(ValidationError, d.save)
        self.assertRaises(ValidationError, d.insert)
        del d.bar
        d.save()
        del d.foo
        self.assertRaises(ValidationError, d.save)

    @unittest.skipUnless(PYMONGO_OK, 'pymongo not installed or connection refused')
    def test_partial_update_addToSet(self):
        """Pymongo: addToSet functionality with `save`"""
        client = pymongo.MongoClient()

        class Doc(BaseDocument):
            dot_notation = True
            foo = Field(list)
            bar = Field(dict)
            moo = Field(six.text_type)
        Doc.register(client=client, db='nanotestdb')

        self.assertRaises(ValidationError, Doc().addToSet, *('$fail', 42))
        self.assertRaises(ValidationError, Doc().addToSet, *('bar.$1', 42))
        # use dict.__setitem__ to bypass RecordingDict cast at self.__setitem__
        d = Doc()
        dict.__setitem__(d, 'bar', {})
        self.assertEqual(dict, type(d.bar))  # not RecordingDict
        self.assertRaises(ValidationError, d.addToSet, *('bar.1', 42))
        d = Doc(foo=['foo_1', 'foo_2'], bar={'1': 'bar_1', '2': []}, moo=six.u('moo val'))
        d.insert()
        self.assertRaises(ValidationError, d.addToSet, *('moo', 42))
        self.assertRaises(ValidationError, d.addToSet, *('moo.not_dict', 42))
        self.assertRaises(ValidationError, d.addToSet, *('undefined.field', 42))
        self.assertRaises(UnsupportedOperation, d.addToSet, *('bar.a.b', 42))
        d.addToSet('foo', 'foo_1')
        d.moo = six.u('new moo')
        d.addToSet('foo', 'foo_3')
        d.addToSet('foo', 'foo_1')
        d.addToSet('bar.2', 'L33t')
        d.addToSet('bar.3', 'new_1')
        d.addToSet('bar.3', 'new_1')
        d.addToSet('bar.3', 'new_2')
        self.assertRaises(ValidationError, d.addToSet, *('bar.1', 1))
        topdiff = {'$set': {'moo': 'new moo'}, '$unset': {},
                   '$addToSet': {'foo': {'$each': ['foo_1', 'foo_3']}}}
        subdiff = {'$set': {}, '$unset': {},
                   '$addToSet': {'2': {'$each': ['L33t']},
                                 '3': {'$each': ['new_1', 'new_2']}}}
        self.assertEqual(topdiff, d.__nanodiff__)
        self.assertEqual(subdiff, d.bar.__nanodiff__)
        d_copy = copy.deepcopy(d)
        d.save()
        d_db = Doc.find_one()
        self.assertTrue(d_copy == d == d_db)
        # check against field duplication at addToSet
        d = Doc()
        d.foo = [42]  # set -- top-level $addToSet will clash
        self.assertEqual([42], d.__nanodiff__['$set']['foo'])
        self.assertRaises(ValidationError, d.addToSet, *('foo', 42))
        del d.foo  # unset -- top-level $addToSet will clash
        self.assertRaises(ValidationError, d.addToSet, *('foo', 42))
        d = Doc(bar={})
        d.bar['1'] = [42]  # deep-level set -- dotted $addToSet will clash
        self.assertRaises(ValidationError, d.addToSet, *('bar.1', 42))
        d = Doc()
        d.bar = {'1': [42]}  # dict set on top-level -- dotted $addToSet will clash
        self.assertEqual({'bar': {'1': [42]}}, d.__nanodiff__['$set'])
        self.assertRaises(ValidationError, d.addToSet, *('bar.1', 42))

        class Doc2(BaseDocument):
            dot_notation = True
            optional = Field(dict, required=False)

        Doc2.register(client=client, db='nanotestdb')
        dd = Doc2()
        dd.insert()
        # addToSet on unset field
        dd.addToSet('optional.sub', 42)
        self.assertEqual([42], dd.optional['sub'])
        self.assertEqual({'sub': {'$each': [42]}}, dd.optional.__nanodiff__['$addToSet'])
        dd.save()
        self.assertEqual(1, Doc2.find({'optional.sub': 42}).count())

    @unittest.skipUnless(PYMONGO_OK, 'pymongo not installed or connection refused')
    def test_sub_diff(self):
        """Test embedded document diff"""
        client = pymongo.MongoClient()

        class Doc(BaseDocument):
            dot_notation = True
            foo = Field(six.text_type)
            bar = Field(dict)
        Doc.register(client=client, db='nanotestdb')

        d = Doc()
        d.foo = six.u('foo value')
        d.bar = {'sub_a': 'a', 'sub_b': 'b'}
        d.insert()
        self.assertEqual(d, Doc.find_one())
        d.foo = six.u('foo value')
        d.bar['sub_b'] = 'b'  # no change, update will return None
        self.assertEqual(None, d.save())
        d.bar = {'sub_a': 'a', 'sub_b': 'b'}
        self.assertEqual(None, d.save())
        del d.bar['sub_a']
        d.bar['sub_c'] = 'c'
        d.foo = six.u('new foo')
        d.save()
        expected = {'_id': d._id, 'foo': 'new foo',
                    'bar': {'sub_b': 'b', 'sub_c': 'c'}}
        self.assertTrue(expected == d == Doc.find_one())

    @unittest.skipUnless(PYMONGO_OK, 'pymongo not installed or connection refused')
    def test_auto_update(self):
        """Test auto_update keyword workings"""
        client = pymongo.MongoClient()

        class Doc(BaseDocument):
            dot_notation = True
            foo = Field(six.text_type)
            bar = Field(datetime.datetime, auto_update=True)
            moo = Field(int, required=False)
        Doc.register(client=client, db='nanotestdb')

        dt = datetime.datetime.utcnow() - datetime.timedelta(hours=1)
        d = Doc(foo=six.u('foo value'), bar=dt)
        d.insert()
        self.assertTrue(datetime.timedelta(hours=1) < d.bar - dt)
        d.moo = 42
        dt_after_insert = d.bar
        d.save()
        self.assertNotEqual(d.bar, dt_after_insert)

    @unittest.skipUnless(PYMONGO_OK, 'pymongo not installed or connection refused')
    def test_document_dbref(self):
        """Test get_dbref functionality"""
        client = pymongo.MongoClient()

        class Doc(BaseDocument):
            pass
        Doc.register(client=client, db='nanotestdb')

        d = Doc()
        self.assertRaises(AssertionError, d.get_dbref)
        d.insert()
        dbref = d.get_dbref()
        self.assertTrue('doc' == dbref.collection and 'nanotestdb' == dbref.database)
        dd = d.get_collection().database.dereference(dbref)
        self.assertEqual(d['_id'], dd['_id'])

    @unittest.skipUnless(PYMONGO_OK, 'pymongo not installed or connection refused')
    @unittest.skipUnless(six.PY2, 'test irrelevant on PY3')
    def test_string_types(self):
        """Test text type case (as mongodb-pymongo returned string type is always unicode)"""
        client = pymongo.MongoClient()

        class Doc(BaseDocument):
            foo = Field(six.binary_type)
            bar = Field(six.text_type)
        Doc.register(client=client, db='nanotestdb')
        d = Doc(foo=six.binary_type('value \xc3\xbc'), bar=six.u('value \xfc'))
        d.insert()
        dd = Doc.find_one(_id=d['_id'])
        self.assertEqual(type(d['foo']), type(dd['foo']))
        self.assertEqual(type(d['bar']), type(dd['bar']))
        self.assertEqual(dd['foo'].decode('utf-8'), dd['bar'])

    def test_dbref_getter_methods(self):
        """Test the creation and function of ``get_<field_name>_field`` methods"""
        client = pymongo.MongoClient()

        class Doc(BaseDocument):
            foo = Field(six.text_type)
            self = Field(bson.DBRef, required=False, document_class='Doc')
            self2 = Field(bson.DBRef, required=False, document_class='test.test_document.Doc')
            other = Field(bson.DBRef, required=False)
        Doc.register(client=client, db='nanotestdb')
        # temporarily attach Doc to module so document class import can find it
        sys.modules[__name__].Doc = Doc

        # to test DBRef document class auto discover
        # different naming so it wont clash with other Doc2 defined here
        class XDoc2(BaseDocument):
            pass
        XDoc2.register(client=client, db='nanotestdb')

        d = Doc(foo=six.text_type('1337'))
        self.assertTrue(hasattr(d, 'get_self_field'))
        self.assertTrue(isinstance(d.get_self_field, types.MethodType))
        self.assertTrue(not hasattr(d, 'get_foo_field'))
        self.assertRaises(DBRefNotSetError, d.get_self_field)
        d.insert()
        dd = XDoc2()
        dd.insert()
        d['self'], d['self2'] = d.get_dbref(), d.get_dbref()
        d['other'] = dd.get_dbref()
        d.save()
        self.assertTrue(d == d.get_self_field() == d.get_self2_field())
        self.assertEqual({'_id': dd['_id']}, d.get_other_field())  # auto discovered
        self.assertEqual(Doc, type(d.get_self_field()))
        self.assertEqual(XDoc2, type(d.get_other_field()))

        # new subclass using same collection as XDoc2 to test undecided discover
        class Doc3(BaseDocument):
            pass
        Doc3.register(client=client, db='nanotestdb', collection='xdoc2')

        self.assertRaises(UnsupportedOperation, d.get_other_field)

        # cleanup
        del sys.modules[__name__].Doc

class IndexTestCase(unittest.TestCase):
    def setUp(self):
        if PYMONGO_OK:
            pymongo.MongoClient().drop_database('nanotestdb')

    def test_index_definitions_bad(self):

        self.assertRaises(TypeError, Index)

        class FooDoc(BaseDocument):
            dot_notation = True
            foo = Field(six.text_type)

        def bad_def_type_1():  # no fields -> no nanomongo -> no __indexes__ allowed
            class Doc(BaseDocument):
                __indexes__ = ''

        def bad_def_type_2():  # bad __indexes__ content
            class Doc(FooDoc):
                __indexes__ = ['foo']

        def bad_def_type_3():  # bad __indexes__ content
            class Doc(FooDoc):
                __indexes__ = [Index(1)]

        def bad_def_type_4():
            class Doc(FooDoc):  # bad __indexes__ content
                __indexes__ = [Index([(1,)])]

        def def_mismatch_1():  # index field not defined
            class Doc(FooDoc):
                __indexes__ = [Index('bar')]

        def def_mismatch_2():  # list form mismatch, first position
            class Doc(FooDoc):
                __indexes__ = [Index([('bar', 1)])]

        def def_mismatch_3():  # list form mismatch, second position
            class Doc(FooDoc):
                __indexes__ = [
                    Index([('foo', pymongo.ASCENDING), ('bar', pymongo.DESCENDING)]),
                ]

        def def_mismatch_4():
            class Doc(BaseDocument):
                foo = Field(six.text_type)
                bar = Field(int)
                __indexes__ = [
                    Index('foo.fail'),
                    Index([('foo.fail', pymongo.ASCENDING), ('bar.fail', pymongo.ASCENDING)]),
                ]

        counts = {'type': 0, 'mismatch': 0}
        for fname, func in locals().items():  # run the above defined functions
            if 'type' in fname:
                counts['type'] += 1
                self.assertRaises(TypeError, func)
            elif 'mismatch' in fname:
                counts['mismatch'] += 1
                self.assertRaises(IndexMismatchError, func)
        self.assertTrue(4 == counts['type'] == counts['mismatch'])

    @unittest.skipUnless(PYMONGO_OK, 'pymongo not installed or connection refused')
    def test_index_definitions(self):
        client = pymongo.MongoClient()

        class Doc(BaseDocument):
            dot_notation = True
            foo = Field(six.text_type)
            bar = Field(int)
            __indexes__ = [
                Index('foo'),
                Index([('bar', pymongo.ASCENDING), ('foo', pymongo.DESCENDING)],
                      unique=True),
            ]
        Doc.register(client=client, db='nanotestdb')

        self.assertEqual(3, len(Doc.get_collection().index_information()))  # 2 + _id

        class Doc2(Doc):
            moo = Field(float)
            __indexes__ = [
                Index('bar'),  # pointless, but index test on superclass field
                Index([('moo', pymongo.DESCENDING), ('foo', pymongo.ASCENDING)]),
            ]
        Doc2.register(client=client, db='nanotestdb')

        self.assertEqual(3, len(Doc2.get_collection().index_information()))  # 2 + _id

        class Doc3(BaseDocument):
            foo = Field(six.text_type)
            __indexes__ = [Index('foo')]

        self.assertTrue(hasattr(Doc3, '__indexes__'))
        Doc3.register(client=client, db='nanotestdb')
        self.assertEqual(2, len(Doc3.get_collection().index_information()))  # 1 + _id
        self.assertFalse(hasattr(Doc3, '__indexes__'))

        class Doc4(BaseDocument):
            # test indexes on dotted keys
            foo = Field(list)
            bar = Field(dict)
            __indexes__ = [
                Index('bar.moo'),
                # compund on two embedded document fields
                Index([('bar.moo', pymongo.ASCENDING), ('bar.zoo', pymongo.ASCENDING)]),
                # compound on embedded document field + list element document field
                Index([('bar.moo', pymongo.ASCENDING), ('foo.whatever', pymongo.ASCENDING)]),
            ]

        Doc4.register(client=client, db='nanotestdb')
        self.assertEqual(4, len(Doc4.get_collection().index_information()))  # 4 + _id



import importlib
import weakref

import pymongo
import six

from bson import ObjectId, DBRef

from .errors import *
from .field import Field
from .util import (
    RecordingDict, DotNotationMixin, valid_client, NanomongoSONManipulator,
    check_spec,
)


def ref_getter_maker(field_name, document_class=None):
    """create dereference methods for given ``field_name`` to be bound
    to Document instances
    """
    def ref_getter(self):
        if field_name not in self or not self[field_name]:
            raise DBRefNotSetError('"%s" field is not set' % field_name)
        dbref = self[field_name]
        if document_class is not None:
            splat = document_class.split('.')
            class_name = splat.pop()
            module = '.'.join(splat) if splat else self.__class__.__module__
            cls = getattr(importlib.import_module(module), class_name)
        else:
            database = dbref.database if dbref.database else self.nanomongo.database
            collection = dbref.collection
            filter_f = (lambda cls: cls.nanomongo.registered and
                        cls.nanomongo.database.name == database and
                        cls.nanomongo.collection == collection)
            classes = list(filter(filter_f, BaseDocument.__subclasses__()))
            if 1 != len(classes):
                err_str = ('can not guess document class for "%s", found: "%s". '
                           'Please provide document_class kwarg to "%s" Field')
                raise UnsupportedOperation(err_str % (dbref, classes, field_name))
            cls = classes.pop()
        # we don't use dereference since BaseDocument.find_one handles type casting nicely
        return cls.find_one(_id=dbref.id)
    return ref_getter


class BasesTuple(tuple):
    pass


class Index(object):
    """A container for clean index definition, it's ``args`` and
    ``kwargs`` are passed to ``pymongo.Collection.create_index``
    """
    def __init__(self, *args, **kwargs):
        if not args:
            raise TypeError('Index: key or list of key-direction tuples expected')
        self.args = args
        self.kwargs = kwargs


class Nanomongo(object):
    """Contains information about the Document it's attached to like
    its fields (which contain validators), db and collection etc and
    provides methods to ease checks
    """
    def __init__(self, fields=None):
        super(Nanomongo, self).__init__()
        if not isinstance(fields, dict):
            raise TypeError('fields kwarg expected of type dict')
        self.fields = fields
        self.classref = None
        self.registered = False
        self.client, self.database, self.collection = None, None, None
        self.transforms = {}  # save auto_update fields so we don't keep looping
        for field_name, field in self.fields.items():
            if hasattr(field, 'auto_update'):
                self.transforms[field_name] = field.auto_update

    @classmethod
    def from_dicts(cls, *args):
        """Create from dict, filtering relevant items"""
        if not args:
            raise TypeError('from_dicts takes at least 1 positional argument')
        fields = {}
        for dct in args:
            if not isinstance(dct, dict):
                raise TypeError('expected input of dictionaries')
            for field_name, field_value in dct.items():
                if isinstance(field_value, Field):
                    fields[field_name] = field_value
        return cls(fields=fields)

    def has_field(self, key):
        """Check existence of field"""
        return key in self.fields

    def list_fields(self):
        """Return a list of strings denoting fields"""
        return sorted(self.fields.keys())

    def validate(self, field_name, value):
        """Validate field input"""
        return self.fields[field_name].validator(value, field_name=field_name)

    def set_client(self, client):
        """Set client, a Client from pymongo or motor expected"""
        if not valid_client(client):
            raise TypeError('pymongo or motor Client expected')
        self.client = client

    def set_db(self, db_string):
        """Set database, string expected"""
        if not db_string or not isinstance(db_string, six.string_types):
            raise TypeError('Exected database string')
        if not self.client:
            raise ConfigurationError('Mongo client not set')
        self.database = self.client[db_string]

    def set_collection(self, col_string):
        """Set collection, string expected"""
        if not col_string or not isinstance(col_string, six.string_types):
            raise TypeError('Expected collection string')
        self.collection = col_string

    def check_config(self):
        """Check if client, database and collection attributes are set"""
        if not self.client:
            raise ConfigurationError('Mongo client not set')
        elif not self.database:
            raise ConfigurationError('database not set')
        elif not self.collection:
            raise ConfigurationError('collection not set')

    def add_son_manipulator(self):
        """add our son manipulator to transform documents coming from mongodb
        to the class we defined, see
        :class:`~nanomongo.util.NanomongoSONManipulator`
        """
        if six.PY2:  # unicode -> str implicit transform for binary_type (str) Fields
            def str_transformer(unicode_str):
                return unicode_str.encode('utf-8')

            str_fields = ((fname, field) for fname, field in self.fields.items()
                          if six.binary_type == field.data_type)
            transforms = dict((fname, str_transformer) for fname, field in str_fields)
        else:
            transforms = None

        manipulator = NanomongoSONManipulator(self.classref(), transforms=transforms)
        self.database.add_son_manipulator(manipulator)

    def register(self, client=None, db_string=None, collection=None):
        """register the class. this is called from defined documents'
        :meth:`~BaseDocument.register()` method. Note that this also
        runs :meth:`~pymongo.collection.Collection.ensure_index()`
        """
        self.set_client(client) if client else None
        self.set_db(db_string) if db_string else None
        self.set_collection(collection) if collection else None
        self.check_config()
        self.add_son_manipulator()
        # indexes
        doc_class = self.classref()
        indexes = doc_class.__indexes__ if hasattr(doc_class, '__indexes__') else []
        for index in indexes:
            self.ensure_index(index)
        if hasattr(doc_class, '__indexes__'):
            delattr(doc_class, '__indexes__')
        # mark as registered
        self.registered = True

    def get_collection(self):
        """Returns collection"""
        self.check_config()
        return self.database[self.collection]

    def ensure_index(self, index):
        """``Collection.ensure_index`` wrapper"""
        return self.get_collection().ensure_index(*index.args, **index.kwargs)


class DocumentMeta(type):
    """Document Metaclass. Generates allowed field set and their validators
    """

    def __new__(cls, name, bases, dct, **kwargs):
        """Check against illegal attributes (eg. ``nanomongo``); get bases
        so we can get their :class:`~nanomongo.field.Field` definitions
        """
        if 'nanomongo' in dct:
            raise TypeError('field name "nanomongo" is not allowed')
        if '__indexes__' in dct and not isinstance(dct['__indexes__'], list):
            raise TypeError('__indexes__: list of Index instances expected')
        use_dot_notation = kwargs.pop('dot_notation') if 'dot_notation' in kwargs else None
        if 'dot_notation' in dct:
            use_dot_notation = dct.pop('dot_notation')
        new_bases = cls._get_bases(bases)
        if use_dot_notation and DotNotationMixin not in new_bases:
            new_bases = (DotNotationMixin,) + new_bases
        return super(DocumentMeta, cls).__new__(cls, name, new_bases, dct)

    def __init__(cls, name, bases, dct, **kwargs):
        """Create the `~nanomongo.document.Nanomongo` for this class and delete
        :class:`~nanomongo.field.Field` attributes. Also sets client, db, collection
        info if provided and runs indexes"""
        super(DocumentMeta, cls).__init__(name, bases, dct)
        if hasattr(cls, 'nanomongo'):
            cls.nanomongo = Nanomongo.from_dicts(cls.nanomongo.fields, dct)
        else:
            cls.nanomongo = Nanomongo.from_dicts(dct)
        if not cls.nanomongo.has_field('_id'):
            cls.nanomongo.fields['_id'] = Field(ObjectId, required=False)
        for field_name, field_value in dct.items():
            if isinstance(field_value, Field):
                delattr(cls, field_name)
        # client, database, collection
        cls.nanomongo.classref = weakref.ref(cls)

        def _check_arg(arg):
            return arg in kwargs or hasattr(cls, arg)

        def _get_arg(arg):
            """get arg from kwargs or from class attribute and
            remove class attribute"""
            if arg in kwargs:
                return kwargs[arg]
            retval = getattr(cls, arg)
            delattr(cls, arg)
            return retval

        if _check_arg('client'):
            cls.nanomongo.set_client(_get_arg('client'))
        if _check_arg('db'):
            cls.nanomongo.set_db(_get_arg('db'))
        if _check_arg('collection'):
            cls.nanomongo.set_collection(_get_arg('collection'))
        else:
            cls.nanomongo.set_collection(name.lower())
        # register if nanomongo config is OK
        try:
            cls.nanomongo.check_config()
            cls.nanomongo.register()
        except ConfigurationError:
            pass
        # indexes
        indexes = cls.__indexes__ if hasattr(cls, '__indexes__') else []

        for index in indexes:
            if not isinstance(index, Index):
                raise TypeError('__indexes__: list of Index instances expected')
            cls.check_index(index)

    def check_index(cls, index):
        """check correctness of :class:`~Index` definitions"""
        def valid_index_key(ikey):
            if '.' not in ikey:
                return cls.nanomongo.has_field(ikey)
            field = ikey.split('.')[0]
            return (cls.nanomongo.has_field(field) and
                    cls.nanomongo.fields[field].data_type in [dict, list])
        i = index.args[0]  # key or list
        if not isinstance(i, (six.string_types, list)):
            raise TypeError('Index: str or list of key-value tuples expected')
        if isinstance(i, six.string_types) and not valid_index_key(i):
            raise IndexMismatchError('field for index "%s" does not exist' % i)
        elif isinstance(i, list):
            for tup in i:
                if not isinstance(tup, tuple) or 2 != len(tup):
                    raise TypeError('Index: list of key-value tuples expected')
                if not valid_index_key(tup[0]):
                    err_str = 'field for index "%s" does not exist' % tup[0]
                    raise IndexMismatchError(err_str)

    @classmethod
    def _get_bases(cls, bases):
        # taken from MongoEngine
        if isinstance(bases, BasesTuple):
            return bases
        seen = []
        bases = cls.__get_bases(bases)
        unique_bases = (b for b in bases if not (b in seen or seen.append(b)))
        return BasesTuple(unique_bases)

    @classmethod
    def __get_bases(cls, bases):
        for base in bases:
            if base is object:
                continue
            yield base
            for child_base in cls.__get_bases(base.__bases__):
                yield child_base


@six.add_metaclass(DocumentMeta)
class BaseDocument(RecordingDict):
    """BaseDocument class. Subclasses should be used. See
    :meth:`~BaseDocument.__init__()`
    """

    def __init__(self, *args, **kwargs):
        """Inits the document with given data and validates the fields
        (field validation bad idea during init?). If you define
        ``__init__`` method for your document class, make sure to call
        this
        ::

            class MyDoc(BaseDocument, dot_notation=True):
                foo = Field(str)
                bar = Field(int, required=False)

                def __init__(self, *args, **kwargs):
                    super(MyDoc, self).__init__(*args, **kwargs)
                    # do other stuff
        """
        # if input dict, merge (not updating) into kwargs
        if args and not isinstance(args[0], dict):
            raise TypeError('dict or dict subclass argument expected')
        elif args:
            for field_name, field_value in args[0].items():
                if field_name not in kwargs:
                    kwargs[field_name] = field_value
        super(BaseDocument, self).__init__()
        for field_name, field in self.nanomongo.fields.items():
            if hasattr(field, 'default_value'):
                val = field.default_value
                dict.__setitem__(self, field_name, val() if callable(val) else val)
            # attach get_<field_name>_field methods for DBRef fields
            if field.data_type in [DBRef] + DBRef.__subclasses__():
                getter_name = 'get_%s_field' % field_name
                doc_class = field.document_class if hasattr(field, 'document_class') else None
                getter = ref_getter_maker(field_name, document_class=doc_class)
                setattr(self, getter_name, six.create_bound_method(getter, self))
        for field_name in kwargs:
            if self.nanomongo.has_field(field_name):
                self.nanomongo.validate(field_name, kwargs[field_name])
                dict.__setitem__(self, field_name, kwargs[field_name])
            else:
                raise ExtraFieldError('Undefined field %s=%s in %s' %
                                      (field_name, kwargs[field_name], self.__class__))
        for field_name, field_value in self.items():
            # transform dict to RecordingDict so we can track diff in embedded docs
            if isinstance(field_value, dict):
                dict.__setitem__(self, field_name, RecordingDict(field_value))

    @classmethod
    def register(cls, client=None, db=None, collection=None):
        """Register this document. Sets client, database, collection
        information, builds (ensure) indexes and sets SON manipulator
        """
        if cls.nanomongo.registered:
            err_str = '''%s is already registered. This is automatic if you have defined
your document class with client, db, collection.''' % cls
            raise ConfigurationError(err_str)
        cls.nanomongo.register(client=client, db_string=db, collection=collection)

    @classmethod
    def get_collection(cls):
        """Returns collection as set in :attr:`~cls.nanomongo`"""
        return cls.nanomongo.get_collection()

    @classmethod
    def find(cls, *args, **kwargs):
        """``pymongo.Collection().find`` wrapper for this document"""
        if args:
            check_spec(cls, args[0])
        return cls.get_collection().find(*args, **kwargs)

    @classmethod
    def find_one(cls, *args, **kwargs):
        """``pymongo.Collection().find_one`` wrapper for this document"""
        if args:
            check_spec(cls, args[0])
        return cls.get_collection().find_one(*args, **kwargs)

    def __dir__(self):
        """Add defined Fields to dir"""
        return sorted(dir(super(BaseDocument, self)) + self.nanomongo.list_fields())

    def validate(self):
        """Override this to add extra document validation, will be
        called at the end of :meth:`~validate_all()` """
        pass

    def validate_all(self):
        """Check against extra fields, run field validators and
        user-defined :meth:`~validate()` """
        for field, value in self.items():
            if not self.nanomongo.has_field(field):
                raise ValidationError('extra field "%s" with value "%s"' % (field, value))
        for field_name, field in self.nanomongo.fields.items():
            if field_name in self:
                field.validator(self[field_name], field_name=field_name)
            elif field.required:
                raise ValidationError('required field "%s" missing' % field_name)
        return self.validate()

    def validate_diff(self):
        """Check correctness of diffs before partial update, also run
        user-defined :meth:`~validate()` """
        sets = self.__nanodiff__['$set']
        unsets = self.__nanodiff__['$unset']
        for field_name, field_value in unsets.items():
            if self.nanomongo.has_field(field_name):
                if self.nanomongo.fields[field_name].required:
                    raise ValidationError('can not unset "%s", required' % field_name)
        for field_name, field_value in sets.items():
            if not self.nanomongo.has_field(field_name):
                err_str = 'extra field "%s" with value "%s"'
                raise ValidationError(err_str % (field_name, field_value))
            field = self.nanomongo.fields[field_name]
            field.validator(field_value, field_name=field_name)
        return self.validate()

    def run_auto_updates(self):
        """Runs functions in :attr:`nanomongo.transforms` like
        auto_update stuff before :meth:`~insert()` :meth:`~save()`
        """
        for field_name, updater in self.nanomongo.transforms.items():
            self[field_name] = updater()

    def insert(self, **kwargs):
        """Insert document into database, return _id. Runs
        :meth:`~run_auto_updates()` and :meth:`~validate_all()` """
        self.run_auto_updates()
        self.validate_all()
        id_or_ids = self.get_collection().insert(self, **kwargs)
        self.reset_diff()
        for field_name, field_value in self.items():
            if isinstance(field_value, dict):  # cast dicts
                field_value = RecordingDict(field_value)
        return id_or_ids

    def save(self, **kwargs):
        """Saves document. This method only does partial updates and no
        inserts. Runs :meth:`~run_auto_updates()` and :meth:`~validate_all()`
        prior to save. Returns ``Collection.update()`` response
        """
        if '_id' not in self:
            raise ValidationError('insert first; save does partial updates')
        if '_id' in self.__nanodiff__['$set']:
            raise ValidationError('_id seems to be manually set, do insert')
        self.run_auto_updates()
        self.validate_diff()
        assert 3 == len(self.__nanodiff__), '__nanodiff__: %s' % self.__nanodiff__
        query = {'_id': self['_id']}
        diff = self.__nanodiff__
        # get subdiff containing dotted keys, merge into diff
        subdiff = self.get_sub_diff()
        diff['$set'].update(subdiff['$set'])
        diff['$unset'].update(subdiff['$unset'])
        diff['$addToSet'].update(subdiff['$addToSet'])
        # remove empty update ops, MongoDB 2.6 returns error for them
        diff = dict(filter(lambda update: update[1], diff.items()))
        # for operation in ('$set', '$unset', '$addToSet'):
        #     if {} == diff[operation]:
        #         del diff[operation]
        if not diff:
            self.reset_diff()
            return
        response = self.get_collection().update(query, diff, **kwargs)
        self.reset_diff()
        return response

    def addToSet(self, field, value):
        """MongoDB ``Collection.update()`` $addToSet functionality.
        This sets the value accordingly and records the change in
        :attr:`~__nanodiff__` to be saved with :meth:`~save()`.
        ::

            # MongoDB style dot notation can be used to add to lists
            # in embedded documents
            doc = Doc(foo=[], bar={})
            doc.addToSet('foo', new_value)
            doc.addToSet('bar.sub_field', new_value)

        Contrary to how $set has no effect under __setitem__ (see
        :class:`~.util.RecordingDict`.__setitem__) when the
        new value is equal to the current; $addToSet explicitly adds
        the call to :attr:`~__nanodiff__` so it will be sent to the
        database when :meth:`save()` is called.
        """
        def top_level_add(self, field, value):
            """add the value to field. appending if the list exists and
            does not contain the value; create new list otherwise.
            raise :class:`.errors.ValidationError` if non-list value initiated
            """
            self.check_can_update('$addToSet', field)
            if field in self and isinstance(self[field], list):
                if value not in self[field]:
                    self[field].append(value)
            elif field not in self or self[field] is None:
                dict.__setitem__(self, field, [value])  # to avoid $set record
            else:
                err_str = 'Could not $addToSet on valid field, bad init? %s: %s'
                raise ValidationError(err_str % (field, self[field]))
            if field not in self.__nanodiff__['$addToSet']:
                self.__nanodiff__['$addToSet'][field] = {'$each': [value]}
            elif value not in self.__nanodiff__['$addToSet'][field]['$each']:
                self.__nanodiff__['$addToSet'][field]['$each'].append(value)

        if field.startswith('$') or '.$' in field:
            err_str = 'MongoDB does not allow fields starting with $. "%s"'
            raise ValidationError(err_str % field)
        # if top-level
        if '.' not in field:
            if ((self.nanomongo.has_field(field) and
                 list == self.nanomongo.fields[field].data_type)):
                top_level_add(self, field, value)  # add & record
            elif self.nanomongo.has_field(field):
                err_str = 'Cannot apply $addToSet modifier to non-array: %s=%s'
                err_str = err_str % (field, self.nanomongo.fields[field].data_type)
                raise ValidationError(err_str)
            else:
                raise ValidationError('Undefined field: "%s"' % top_key)
        # if deep-level
        else:
            try:
                top_key, deep_key = field.split('.')
            except ValueError:
                err_str = '''Only top level and one level deep keus supported for \
$addToSet: "%s"'''
                raise UnsupportedOperation(err_str, field)
            if not self.nanomongo.has_field(top_key):
                raise ValidationError('Undefined field: "%s"' % top_key)
            elif dict != self.nanomongo.fields[top_key].data_type:
                raise ValidationError('"%s" is not a dict' % top_key)
            # field name ok, ensure top level value is RecordingDict type
            if top_key not in self:  # not set yet, do it
                dict.__setitem__(self, top_key, RecordingDict())
            elif not isinstance(self[top_key], RecordingDict):
                # what did you do, use dict.__setitem__ ? :)
                err_str = '''Dotted key's target is not a RecordingDict: %s=%s \
If you've just set it as a new dict; FYI: you can't $set and $addToSet together'''
                raise ValidationError(err_str % (top_key, self[top_key]))
            # make sure we have no $set or $unset on top_key
            self.check_can_update('$addToSet', top_key)
            top_level_add(self[top_key], deep_key, value)  # add & record

    def get_dbref(self):
        """create a ``bson.DBRef`` instance for this :class:`BaseDocument`
        instance
        """
        assert '_id' in self and self['_id'], 'Cannot get DBRef for document with no _id'
        collection = self.get_collection()
        return DBRef(collection.name, self['_id'], database=collection.database.name)



import unittest

from mock import patch

from nanomongo.field import Field
from nanomongo.document import BaseDocument
from nanomongo.util import (
    DotNotationMixin, valid_field, valid_client, RecordingDict, check_keys,
    allow_client,
)
from nanomongo.errors import ValidationError

try:
    import pymongo
    PYMONGO_CLIENT = pymongo.MongoClient()
except:
    PYMONGO_CLIENT = False

try:
    import motor
    MOTOR_CLIENT = motor.MotorClient()
except:
    MOTOR_CLIENT = None


class HelperFuncionsTestCase(unittest.TestCase):
    def test_invalid_clients(self):
        invalid_clients = ['foo', 42, dict(), list(), object(), tuple()]
        [self.assertFalse(valid_client(client)) for client in invalid_clients]

    @unittest.skipUnless(PYMONGO_CLIENT, 'pymongo not installed or connection refused')
    def test_valid_client_pymongo(self):
        client = pymongo.MongoClient()
        self.assertTrue(valid_client(client))

    @unittest.skipUnless(MOTOR_CLIENT, 'motor not installed or connection refused')
    def test_valid_client_motor(self):
        self.assertTrue(valid_client(MOTOR_CLIENT))

    def test_check_keys(self):
        bad_dicts = [
            {'foo.bar': 42}, {'$foo': 42}, {'foo': {'bar.foo': 42}},
            {'foo': {'$bar': 42}},
        ]
        [self.assertRaises(ValidationError, check_keys, *(dct,)) for dct in bad_dicts]
        self.assertRaises(TypeError, check_keys, *([],))

    @unittest.skipUnless(PYMONGO_CLIENT, 'pymongo not installed or connection refused')
    def test_check_spec(self):
        """Test check spec and warning messages"""
        class Doc(BaseDocument):
            client = PYMONGO_CLIENT
            db = 'nanotestdb'
            foo = Field(str)

        with patch('nanomongo.util.logging') as mock_logging:
            Doc.find({'bar': 42})
            for level in ('debug', 'info', 'error', 'critical'):
                self.assertFalse(getattr(mock_logging, level).called)
            self.assertTrue(mock_logging.warn.called)
            Doc.find({'foo.bar': 42})
            self.assertTrue(mock_logging.warn.called)
            Doc.find_one({'bar.moo': 42})
            self.assertTrue(mock_logging.warn.called)

    def test_allow_mock(self):
        class MockClient():
            pass

        client = MockClient()
        self.assertFalse(valid_client(client))
        allow_client(MockClient)
        self.assertTrue(valid_client(client))


class RecordingDictTestCase(unittest.TestCase):
    def test_recording_dict(self):
        """Test `__setitem__`, `__delitem__` functionality"""
        class Doc(RecordingDict):
            pass

        d = Doc()
        nanodiff_base = {'$set': {}, '$unset': {}, '$addToSet': {}}
        self.assertEqual(nanodiff_base, d.__nanodiff__)
        d.foo = 42
        self.assertEqual(nanodiff_base, d.__nanodiff__)
        d['foo'] = 42
        self.assertEqual({'$set': {'foo': 42}, '$unset': {}, '$addToSet': {}}, d.__nanodiff__)
        d['foo'] = 1337
        self.assertEqual({'$set': {'foo': 1337}, '$unset': {}, '$addToSet': {}}, d.__nanodiff__)
        d.reset_diff()
        self.assertEqual(nanodiff_base, d.__nanodiff__)
        d = Doc(foo=42)
        self.assertEqual(nanodiff_base, d.__nanodiff__)
        self.assertEqual(42, d['foo'])
        del d['foo']
        self.assertEqual({'$set': {}, '$unset': {'foo': 1}, '$addToSet': {}}, d.__nanodiff__)
        d['foo'] = "L33t"
        self.assertEqual({'$set': {'foo': "L33t"}, '$unset': {}, '$addToSet': {}}, d.__nanodiff__)

    def test_sub_diff(self):
        """Test functionality of sub_diff for embedded documents"""
        class Doc(RecordingDict):
            pass

        nanodiff_base = {'$set': {}, '$unset': {}, '$addToSet': {}}
        d = Doc()
        self.assertEqual(nanodiff_base, d.__nanodiff__)
        d['sub'] = Doc()
        self.assertEqual({'$set': {'sub': {}}, '$unset': {}, '$addToSet': {}}, d.__nanodiff__)
        d.reset_diff()
        d['sub']['foo'] = 42
        self.assertEqual(nanodiff_base, d.__nanodiff__)  # no diff on top level
        self.assertEqual({'$set': {'foo': 42}, '$unset': {}, '$addToSet': {}}, d['sub'].__nanodiff__)
        subdiff = d.get_sub_diff()
        self.assertEqual({'$set': {'sub.foo': 42}, '$unset': {}, '$addToSet': {}}, subdiff)
        del d['sub']['foo']
        d['sub']['bar'] = 1337
        expected = {'$set': {'sub.bar': 1337}, '$unset': {'sub.foo': 1}, '$addToSet': {}}
        self.assertEqual(expected, d.get_sub_diff())
        d.reset_diff()
        d['sub']['bar'] = 1337  # same value set
        self.assertEqual(nanodiff_base, d.get_sub_diff())


class MixinTestCase(unittest.TestCase):
    def test_mixin(self):
        """Test dot_notation mixin"""
        class Doc(BaseDocument, DotNotationMixin):
            foo = Field(str)

        d = Doc()
        self.assertTrue(valid_field(d, 'foo'))
        self.assertRaises(AttributeError, lambda: d.foo)
        d['foo'] = 'bar'
        self.assertTrue('bar' == d['foo'] == d.foo)
        d.foo = 'foobar'
        self.assertTrue('foobar' == d['foo'] == d.foo)
        self.assertRaises(AttributeError, lambda: d.undefined)
        d.undefined = 42
        self.assertEqual(42, d.undefined)
        self.assertRaises(KeyError, lambda: d['undefined'])



import __main__
import logging
import warnings

import pymongo

from pymongo import son_manipulator

from .errors import ExtraFieldError, ValidationError

ok_types = ()

try:
    import pymongo
    ok_types += (pymongo.MongoClient, pymongo.MongoReplicaSetClient)
    import motor
    ok_types += (motor.MotorClient, motor.MotorReplicaSetClient)
except ImportError as e:
    if not ok_types:
        raise e

logging.basicConfig(format='[%(asctime)s] %(levelname)s [%(module)s.%(funcName)s():%(lineno)d] %(message)s')
logger = logging.getLogger(__file__)


def valid_client(client):
    """returns ``True`` if input is pymongo or motor client
    or any client added with allow_client()"""
    return isinstance(client, ok_types)


def allow_client(client_type):
    """Allows another type to act as client type.
    Intended for using with mock clients."""
    global ok_types
    ok_types += (client_type,)


def valid_field(obj, field):
    return object.__getattribute__(obj, 'nanomongo').has_field(field)


def check_keys(dct):
    """Recursively check against '.' and '$' at start position in
    dictionary keys
    """
    if not isinstance(dct, dict):
        raise TypeError('dict-like argument expected')
    dot_err_str = 'MongoDB does not allow . in field names. "%s"'
    dollar_err_str = 'MongoDB does not allow fields starting with $. "%s"'
    for k, v in dct.items():
        if '.' in k:
            raise ValidationError(dot_err_str % k)
        elif k.startswith('$'):
            raise ValidationError(dollar_err_str % k)
        elif isinstance(v, dict):
            check_keys(v)


def check_spec(cls, spec):
    """Check the query spec for given class & display warnings.
    Dotted keys are checked for top-level field existence and its type
    being dict/list. Normal keys are checked for field existence only.
    """
    w_field = '{0} has no field "{1}", can not match'
    w_field_type = '{0} field "{1}" is not of type {2}, can not match'
    for field in spec.keys():
        f = field.split('.')[0]
        if not cls.nanomongo.has_field(f):
            logging.warn(w_field.format(cls, f))
        elif '.' in field:
            dtype = cls.nanomongo.fields[f].data_type
            if dtype not in (dict, list):
                logging.warn(w_field_type.format(cls, f, (dict, list)))


class RecordingDict(dict):
    """A dictionary subclass modifying :meth:`~__setitem__()` and
    :meth:`~__delitem__()` methods to record changes in its
    :attr:`~__nanodiff__` attribute"""
    def __init__(self, *args, **kwargs):
        super(RecordingDict, self).__init__(*args, **kwargs)
        self.__nanodiff__ = {
            '$set': {}, '$unset': {}, '$addToSet': {},
        }

    def __setitem__(self, key, value):
        """Override :meth:`~dict.__setitem__` so we can track changes"""
        try:
            skip = self[key] == value
        except KeyError:
            skip = False
        if skip:
            return
        value = RecordingDict(value) if isinstance(value, dict) else value
        super(RecordingDict, self).__setitem__(key, value)
        self.__nanodiff__['$set'][key] = value
        self.clean_other_modifiers('$set', key)

    def __delitem__(self, key):
        """Override :meth:`~dict.__delitem__` so we can track changes"""
        super(RecordingDict, self).__delitem__(key)
        self.__nanodiff__['$unset'][key] = 1
        self.clean_other_modifiers('$unset', key)

    def clean_other_modifiers(self, current_mod, field_name):
        """Given `current_mod`, removes other `field_name` modifiers,
        eg. when called with `$set`, removes `$unset` and `$addToSet`
        etc. on `field_name`
        """
        for mod, updates in self.__nanodiff__.items():
            if mod == current_mod:
                continue
            if field_name in updates:
                del self.__nanodiff__[mod][field_name]

    def reset_diff(self):
        """reset `__nanodiff__` recursively; to be used after saving
        diffs. This does NOT do a rollback. Reload from db for that
        """
        nanodiff_base = {'$set': {}, '$unset': {}, '$addToSet': {}}
        self.__nanodiff__ = nanodiff_base
        for field_name, field_value in self.items():
            if isinstance(field_value, RecordingDict):
                field_value.reset_diff()

    def get_sub_diff(self):
        """get `__nanodiff__` from embedded documents. Find fields of
        :class:`~RecordingDict` type, iterate over their diff and build dotted
        keys for top level diff
        """
        diff = {'$set': {}, '$unset': {}, '$addToSet': {}}
        for field_name, field_value in self.items():
            if isinstance(field_value, RecordingDict):
                sets = field_value.__nanodiff__['$set']
                unsets = field_value.__nanodiff__['$unset']
                addtosets = field_value.__nanodiff__['$addToSet']
                for k, v in sets.items():
                    dotkey = '%s.%s' % (field_name, k)
                    diff['$set'][dotkey] = v
                for k, v in unsets.items():
                    dotkey = '%s.%s' % (field_name, k)
                    diff['$unset'][dotkey] = v
                for k, v in addtosets.items():
                    dotkey = '%s.%s' % (field_name, k)
                    diff['$addToSet'][dotkey] = v
        return diff

    def check_can_update(self, modifier, field_name):
        """Check if given `modifier` `field_name` combination can be
        added. MongoDB does not allow field duplication with update
        modifiers. This is to be used with methods `addToSet` ...
        """
        for mod, updates in self.__nanodiff__.items():
            if mod == modifier:
                continue
            if field_name in updates:
                err_str = 'Field name duplication not allowed with modifiers '
                err_str += ('new: {%s} old: {%s: {%s: %s}}' %
                            (modifier, mod, field_name, updates[field_name]))
                raise ValidationError(err_str)


class DotNotationMixin(object):
    """Mixin to make dot notation available on dictionaries"""

    # TODO: When dot_notation is active but key not a field, FAIL?
    def __setattr__(self, key, value):
        """object attribute setting eg. ``self.foo = 42``"""
        if not valid_field(self, key):
            super(DotNotationMixin, self).__setattr__(key, value)
        else:
            self.__setitem__(key, value)

    def __getattr__(self, key):
        """object attribute lookup eg. ``print(self.foo)``"""
        if not valid_field(self, key):
            return super(DotNotationMixin, self).__getattribute__(key)
        try:
            return self.__getitem__(key)
        except KeyError:
            pass
        raise AttributeError("'%s' object has no attribute '%s'" %
                             (self.__class__.__name__, key))

    def __getattribute__(self, key):
        # first to check interpreter
        if hasattr(__main__, '__file__') or not valid_field(self, key):
            return super(DotNotationMixin, self).__getattribute__(key)
        try:
            return self.__getitem__(key)
        except:
            return

        return super(DotNotationMixin, self).__getattribute__(key)

    def __delattr__(self, key):
        """object attribute delete eg. ``del self.foo``"""
        if not valid_field(self, key):
            super(DotNotationMixin, self).__delattr__(key)
            return
        try:
            self.__delitem__(key)
        except KeyError:
            raise AttributeError("'%s' object has no attribute '%s'" %
                                 (self.__class__.__name__, key))


class NanomongoSONManipulator(pymongo.son_manipulator.SONManipulator):
    """A pymongo SON Manipulator used on data that comes from the database
    to transform data to the document class we want because `as_class`
    argument to pymongo find methods is called in a way that screws us.

    - Recursively applied, we don't want that
    - `__init__` is not properly used but rather __setitem__, fails us

    JIRA: PYTHON-175 PYTHON-215
    """
    def __init__(self, as_class, transforms=None):
        self.as_class = as_class
        if transforms:
            assert isinstance(transforms, dict), 'transforms must be a dict'
            self.transforms = transforms

    def will_copy(self):
        return True

    def transform_outgoing(self, son, collection):
        if hasattr(self, 'transforms'):
            for field, transformer in self.transforms.items():
                son[field] = transformer(son[field])
        try:
            return self.as_class(son)
        except ExtraFieldError:
            return son



import copy
import datetime

import six

from bson import DBRef, ObjectId

from .errors import ValidationError
from .util import check_keys


class Field(object):
    """Instances of this class is used to define field types and automatically
    create validators. Note that a Field definition has no value added::

        field_name = Field(str, default='cheeseburger')
        foo = Field(datetime, auto_update=True)
        bar = Field(list, required=False)

    """
    allowed_types = (bool, int, float, six.binary_type, six.text_type,
                     list, dict, datetime.datetime, DBRef, ObjectId)
    # kwarg_name : kwarg_input_validator dictionaries
    allowed_kwargs = {
        'default': lambda v: True,
        'required': lambda v: isinstance(v, bool),
    }
    extra_kwargs = {
        datetime.datetime: {'auto_update': lambda v: isinstance(v, bool)},
        DBRef: {'document_class': lambda v: isinstance(v, six.string_types)},
    }

    def __init__(self, *args, **kwargs):
        """Field kwargs are checked for correctness and field validator is set,
        along with other attributes such as ``required`` and ``auto_update``

        :Keyword Arguments:
          - `default`: default field value, must pass type check, can be a ``callable``
          - `required`: if ``True`` field must exist and not be ``None`` (default: ``True``)
          - `auto_update`: set value to ``datetime.utcnow()`` before inserts/saves;
            only valid for datetime fields (default: ``False``)

        """
        if not args:
            raise TypeError('Field definition incorrect, please provide type')
        elif not isinstance(args[0], type):
            raise TypeError('Field input not a type')
        self.data_type = args[0]
        if ((self.data_type not in self.allowed_types and
             not issubclass(self.data_type, self.allowed_types))):
            raise TypeError('Field input type %s is not allowed' % self.data_type)
        self.check_kwargs(kwargs, self.data_type)
        # attributes
        if 'auto_update' in kwargs and kwargs['auto_update']:
            self.auto_update = self.data_type.utcnow  # datetime.datetime
        if 'document_class' in kwargs and kwargs['document_class']:
            self.document_class = kwargs['document_class']
        self.validator = self.generate_validator(self.data_type, **kwargs)
        self.required = kwargs['required'] if 'required' in kwargs else True
        if 'default' in kwargs:
            self.default_value = kwargs['default']
            if not callable(self.default_value):
                validation_failed = False
                try:
                    self.validator(self.default_value)
                except ValidationError as e:
                    new_err = ('default value "%s"' % kwargs['default']) + ''.join(e.args)
                    validation_failed = True
                if validation_failed:
                    raise TypeError(new_err)
                # check if dict/list type and wrap copy in callable
                if isinstance(self.default_value, (dict, list)):
                    def default_value_wrapper():
                        return copy.deepcopy(kwargs['default'])
                    self.default_value = default_value_wrapper

    @classmethod
    def check_kwargs(cls, kwargs, data_type):
        """Check keyword arguments & their values given to ``Field``
        constructor such as ``default``, ``required`` ...
        """
        err_str = data_type.__name__ + ': %s keyword argument not allowed or "%s" value invalid'
        for k, v in kwargs.items():
            # kwargs allowed for all data_types
            if k in cls.allowed_kwargs:
                if not cls.allowed_kwargs[k](v):  # run kwarg validator
                    raise TypeError(err_str % (k, v))
            # type specific kwargs
            elif data_type in cls.extra_kwargs and k in cls.extra_kwargs[data_type]:
                if not cls.extra_kwargs[data_type][k](v):  # run kwarg validator
                    raise TypeError(err_str % (k, v))
            else:
                raise TypeError(err_str % (k, v))

    def generate_validator(self, t, **kwargs):
        """Generates and returns validator function ``(value_to_check, field_name='')``.
        ``field_name`` kwarg is optional, used for better error reporting.
        """
        def validator(val, field_name=''):
            if val is None and 'required' in kwargs and not kwargs['required']:
                return True
            elif val is None:
                raise ValidationError('%s: None is not allowed (field required)' % field_name)
            if not isinstance(val, t):
                raise ValidationError('%s: "%s" not an instance of %s but an instance of %s' %
                                      (field_name, val, t, type(val)))
            if isinstance(val, dict):
                check_keys(val)  # check against . & $ in keys
            return True
        return validator



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import multiprocessing  # http://bugs.python.org/issue15881#msg170215

import nanomongo
import six

from setuptools import setup

setup(
    name='nanomongo',
    version=nanomongo.__version__,
    description='Minimal Python ODM for MongoDB',
    long_description=open('README.rst').read(),
    author=six.u('Eren Güven'),
    author_email='erenguven0@gmail.com',
    url='https://github.com/eguven/nanomongo',
    install_requires=['pymongo >= 2.5', 'six'],
    packages=['nanomongo'],
    setup_requires=['nose'],
    test_suite='nose.main',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Database',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)



import unittest
import time

import pymongo
import six
import tornado.testing

from bson.objectid import ObjectId

from nanomongo.field import Field
from nanomongo.document import Index, BaseDocument

try:
    import motor
    MOTOR_CLIENT = motor.MotorClient()
except:
    MOTOR_CLIENT = None


class MotorDocumentTestCase(tornado.testing.AsyncTestCase):

    def setUp(self):
        self.io_loop = tornado.ioloop.IOLoop.current()
        pymongo.MongoClient().drop_database('nanotestdb')

    @unittest.skipUnless(MOTOR_CLIENT, 'motor not installed or connection refused')
    @tornado.testing.gen_test
    def test_insert_find_motor(self):
        """Motor: Test document save, find, find_one"""

        MOTOR_CLIENT = motor.MotorClient(io_loop=self.io_loop)

        class Doc(BaseDocument):
            dot_notation = True
            foo = Field(six.text_type)
            bar = Field(int, required=False)
        Doc.register(client=MOTOR_CLIENT, db='nanotestdb')

        col = Doc.get_collection()
        self.assertTrue(isinstance(col, motor.MotorCollection))
        result = yield motor.Op(Doc.find_one)
        self.assertEqual(None, result)
        d = Doc(foo=six.u('foo value'), bar=42)
        _id = yield motor.Op(d.insert)
        self.assertTrue(isinstance(_id, ObjectId))
        result = yield motor.Op(Doc.find({'foo': 'foo value'}).count)
        self.assertEqual(1, result)
        result = yield motor.Op(Doc.find_one, {'bar': 42})
        self.assertEqual(d, result)

    @unittest.skipUnless(MOTOR_CLIENT, 'motor not installed or connection refused')
    @tornado.testing.gen_test
    def test_partial_update(self):
        """Motor: partial atomic update with save"""

        class Doc(BaseDocument):
            dot_notation = True
            foo = Field(six.text_type)
            bar = Field(int, required=False)
            moo = Field(list)

        Doc.register(client=MOTOR_CLIENT, db='nanotestdb')

        d = Doc(foo=six.u('foo value'), bar=42)
        d.moo = []
        yield motor.Op(d.insert)
        del d.bar  # unset
        yield motor.Op(d.save)
        result = yield motor.Op(Doc.find_one, {'_id': d._id})
        self.assertEqual(d, result)
        d.foo = six.u('new foo')
        d['bar'] = 1337
        d.moo = ['moo 0']
        yield motor.Op(d.save, atomic=True)
        result = yield motor.Op(Doc.find_one, {'foo': six.u('new foo'), 'bar': 1337})
        self.assertEqual(d, result)
        d.moo = []
        del d['bar']
        yield motor.Op(d.save)
        result = yield motor.Op(Doc.find_one, {'_id': d._id})
        self.assertEqual(d, result)

    @unittest.skipUnless(MOTOR_CLIENT, 'motor not installed or connection refused')
    @tornado.testing.gen_test
    def test_index_motor(self):
        """Motor: test index build using motor"""

        class Doc(BaseDocument):
            foo = Field(six.text_type)
            __indexes__ = [Index('foo')]

        Doc.register(client=MOTOR_CLIENT, db='nanotestdb')
        # bit of a workaround, checking the current_op on the database because
        # register runs ensure_index async and returns; we end up checking Index
        # before it finishes building on slow systems (travis-ci holla!)
        op = yield motor.Op(Doc.get_collection().database.current_op)
        while op['inprog']:
            yield tornado.gen.Task(self.io_loop.call_later, 0.1)
            op = yield motor.Op(Doc.get_collection().database.current_op)
        indexes = yield motor.Op(Doc.get_collection().index_information)
        self.assertEqual(2, len(indexes))  # 1 + _id
        self.assertFalse(hasattr(Doc, '__indexes__'))
